using System;
using System.Data;
using System.Data.OleDb;
using IServer;
using System.Threading;

namespace HbDBF
{
	public class DbFacade : System.MarshalByRefObject, IDbFacade
	{
		#region fields
		private OleDbConnection con;
		private OleDbCommand cmd;
		private string connectionStr = "Provider=Microsoft.Jet.OLEDB.4.0; Data Source=Hotel.mdb";  
		private int naesteGaesteId;
		private int naesteBookingId;
		private int naesteBookingDagId;
		#endregion

		#region properties
		public int NaesteGaesteId
		{
			get{return this.naesteGaesteId;}
			set{this.naesteGaesteId = value;}
		}
		public int NaesteBookingId
		{
			get{return this.naesteBookingId;}
			set{this.naesteBookingId = value;}
		}
		public int NaesteBookingDagId
		{
			get{return this.naesteBookingDagId;}
			set{this.naesteBookingDagId = value;}
		}
		#endregion
		
		#region constructors
		public DbFacade()
		{
			// Find nste gsteID
			con = new OleDbConnection(connectionStr);
			cmd = new OleDbCommand();
			cmd.Connection = con;
			con.Open();
			cmd.CommandText = "SELECT MAX(g.GaestId), MAX(b.BookingId), MAX(bd.BookingDagId) FROM Gaest AS g, Booking AS b, BookingDag AS bd";
			OleDbDataReader reader;
			reader = cmd.ExecuteReader();
			reader.Read();
			try 
			{
				this.NaesteGaesteId = reader.GetInt32(0);
				this.NaesteBookingId = reader.GetInt32(1);
				this.NaesteBookingDagId = reader.GetInt32(2);
			}
			catch (Exception ex)
			{
				Console.WriteLine(ex.ToString());
			}
			con.Close();
		}
		#endregion

		#region methods
		
		#region Booking relaterede metoder		
		public bool GemBooking(int gaestId, int vaerelsesNr, DateTime fraDato, DateTime tilDato)
		{
			// find antal dage
			TimeSpan ts = tilDato - fraDato;
			int antalDage = ts.Days;
			antalDage++;
			
			con.Open();
			
			OleDbTransaction trans = null;
			
			try
			{
				trans = con.BeginTransaction();
				cmd.Transaction = trans;
				
				// opret booking
				cmd.CommandText = "INSERT INTO Booking(BookingId, GaestId, FraDato, TilDato) VALUES (@bookingId, @gaestId, @FraDato, @tilDato)";
				cmd.Parameters.Clear();
				int bId = Interlocked.Increment(ref this.naesteBookingId);
				cmd.Parameters.Add("@bookingId", OleDbType.Integer).Value = bId;
				cmd.Parameters.Add("@gaestId", OleDbType.Integer).Value = gaestId;
				cmd.Parameters.Add("@fraDato", OleDbType.DBDate).Value = fraDato;
				cmd.Parameters.Add("@tilDato", OleDbType.DBDate).Value = tilDato;
				cmd.ExecuteNonQuery();

				// opret bookingdage
				for (int i = 0; i < antalDage; i++)
				{
					cmd.CommandText = "INSERT INTO BookingDag(BookingDagId, VaerelsesNr, Dato, BookingId) VALUES (@bookingDagId, vaerelsesNr, dato, bookingId)";
					cmd.Parameters.Clear();
					int bdId = Interlocked.Increment(ref this.naesteBookingDagId);
					cmd.Parameters.Add("@bookingDagId", OleDbType.Integer).Value = bdId;
					cmd.Parameters.Add("@vaerelsesNr", OleDbType.Integer).Value = vaerelsesNr;
					cmd.Parameters.Add("@dato", OleDbType.DBDate).Value = fraDato.AddDays(i);
					cmd.Parameters.Add("@bookingId", OleDbType.Integer).Value = bId;
					cmd.ExecuteNonQuery();
				}
				trans.Commit();
			}
			catch
			{
				trans.Rollback();
				con.Close();
				return false;
			}
			con.Close();
			return true;
		}
		public bool GemBooking(string navn, string firma, string telefonnr, int vaerelsesNr, DateTime fraDato, DateTime tilDato)
		{
			this.GemGaest(navn, firma, telefonnr);
			DataTable dt = this.FindGaest(navn, firma, telefonnr);
			return this.GemBooking((int)dt.Rows[0][0],vaerelsesNr, fraDato, tilDato);
			
		}
		public DataTable HentLedigeVaerelser(DateTime fraDato, DateTime tilDato)
		{
			/*con = new OleDbConnection(connectionStr);
			cmd = new OleDbCommand();
			cmd.Connection = con;*/
			con.Open();
			string sTilDato = "" + tilDato.Day + "/" + tilDato.Month + "/" + tilDato.Year;
			string sFraDato = "" + fraDato.Day + "/" + fraDato.Month + "/" + fraDato.Year;
			cmd.CommandText = "SELECT DISTINCT v1.Vaerelsesnr, v1.Type, v1.Ikkeryger, v1.Handicap FROM Vaerelse AS v1 WHERE (((v1.Vaerelsesnr) Not In (SELECT DISTINCT v.vaerelsesnr FROM vaerelse AS v, bookingdag AS bd WHERE v.vaerelsesnr=bd.vaerelsesnr And v.Aktiv=True And bd.dato>=#" 
				+ sFraDato + "# And bd.dato<=#" + sTilDato + "#)));";
			Console.WriteLine(cmd.CommandText);
			//cmd.CommandText = "SELECT v.VaerelsesNr, [Type], [IkkeRyger], [Handicap] FROM Vaerelse AS v, BookingDag AS b WHERE Aktiv=True And v.vaerelsesNr=b.vaerelsesNr And (b.dato<#" + sFraDato + "# Or b.dato>#" + sTilDato + "#);";
			DataTable dt = new DataTable();
			try 
			{
				OleDbDataAdapter odb = new OleDbDataAdapter(cmd.CommandText,con);	
				odb.Fill(dt);
				
			}
			catch (Exception ex)
			{
				Console.WriteLine(ex.ToString());
				Console.ReadLine();
			}
			con.Close();
			return dt;
			
		}
		public DataTable HentLedigeVaerelser(DateTime fraDato, DateTime tilDato, int type, int ikkeryger, int handicap)
		{
			/*con = new OleDbConnection(connectionStr);
			cmd = new OleDbCommand();
			cmd.Connection = con;*/
			con.Open();
			string sTilDato = "" + tilDato.Day + "/" + tilDato.Month + "/" + tilDato.Year;
			string sFraDato = "" + fraDato.Day + "/" + fraDato.Month + "/" + fraDato.Year;

			cmd.CommandText = "SELECT DISTINCT v1.Vaerelsesnr, v1.Type, v1.Ikkeryger, v1.Handicap, vt.pris FROM Vaerelse AS v1, Vaerelsestype AS vt WHERE v1.type = vt.vaerelsestype";
			if (type != 2)
			{cmd.CommandText += " AND v1.type = " + type;}
			if (ikkeryger != 2)
			{cmd.CommandText += " AND v1.ikkeryger = " + (ikkeryger == 1);}
			if (handicap != 2)
			{cmd.CommandText += " AND v1.handicap = " + (handicap == 1);}
			cmd.CommandText += " AND ((v1.Vaerelsesnr) Not In (SELECT DISTINCT v.vaerelsesnr FROM vaerelse AS v, bookingdag AS bd WHERE v.vaerelsesnr=bd.vaerelsesnr And v.Aktiv=True And bd.dato>=#"
				+ sFraDato + "# And bd.dato<=#" + sTilDato + "#));";
			Console.WriteLine(cmd.CommandText);
			//cmd.CommandText = "SELECT v.VaerelsesNr, [Type], [IkkeRyger], [Handicap] FROM Vaerelse AS v, BookingDag AS b WHERE Aktiv=True And v.vaerelsesNr=b.vaerelsesNr And (b.dato<#" + sFraDato + "# Or b.dato>#" + sTilDato + "#);";
			DataTable dt = new DataTable();
			try 
			{
				OleDbDataAdapter odb = new OleDbDataAdapter(cmd.CommandText,con);	
				odb.Fill(dt);
				
			}
			catch (Exception ex)
			{
				Console.WriteLine(ex.ToString());
				//Console.ReadLine();
			}
			con.Close();
			return dt; /*
			if (dt.Rows.Count == 0)
			{return null;}
			Varelse v = new Varelse((int)dt.Rows[0][0],(int)dt.Rows[0][1],(bool)dt.Rows[0][2],(bool)dt.Rows[0][3],(int)dt.Rows[0][4]);
			Console.WriteLine(v.Ikkeryger.ToString() + " " + v.Pris);
			return v;*/
			
		}

		#endregion

		#region Bookingoversigt metode
		
		public DataTable BookingOversigt(DateTime fraDato, DateTime tilDato)
		{
			/*con = new OleDbConnection(connectionStr);
			cmd = new OleDbCommand();
			cmd.Connection = con;*/
			con.Open();
			string sTilDato = "" + tilDato.Day + "/" + tilDato.Month + "/" + tilDato.Year;
			string sFraDato = "" + fraDato.Day + "/" + fraDato.Month + "/" + fraDato.Year;
			cmd.CommandText = "SELECT DISTINCT g.Navn, g.Telefonnr, v.Vaerelsesnr, b.FraDato, b.TilDato, vt.Pris FROM Gaest AS g, Booking AS b, Bookingdag AS bd, Vaerelse AS v, Vaerelsestype AS vt WHERE bd.Dato>=#" 
				+ sFraDato + "# And bd.Dato<=#" + sTilDato + "# And bd.Bookingid=b.Bookingid And b.Gaestid=g.GaestId And bd.Vaerelsesnr=v.Vaerelsesnr And v.Type=vt.Vaerelsestype;";
			Console.WriteLine(cmd.CommandText);
			DataTable dt = new DataTable();
			try 
			{
				OleDbDataAdapter odb = new OleDbDataAdapter(cmd.CommandText,con);	
				odb.Fill(dt);
				
			}
			catch (Exception ex)
			{
				Console.WriteLine(ex.ToString());
				Console.ReadLine();
			}
			con.Close();
			return dt;
		}

		#endregion

		#region Metoder til gst
		public DataTable FindGaest(string navn, string firma, string tlf)
		{
			/*con = new OleDbConnection(connectionStr);
			cmd = new OleDbCommand();
			cmd.Connection = con;*/
			con.Open();
			cmd.CommandText = "SELECT GaestId, Navn, Firma, TelefonNr FROM Gaest WHERE Aktiv = TRUE ";
			if (navn != "")
			{ cmd.CommandText += "AND Navn LIKE '%"+navn+"%' "; }
			if (firma != "")
			{ cmd.CommandText += "AND Firma LIKE '%"+firma+"%' "; }
			if (tlf != "")
			{ cmd.CommandText += "AND TelefonNr LIKE '%"+tlf+"%' "; }
			cmd.CommandText += ";";
			DataTable dt = new DataTable();
			try 
			{
				OleDbDataAdapter odb = new OleDbDataAdapter(cmd.CommandText,con);	
				odb.Fill(dt);
				
			}
			catch (Exception ex)
			{
				Console.WriteLine(ex.ToString());
			}
			con.Close();
			return dt;
		}
        
		public bool GemGaest(string navn, string firma, string tlfNr)
		{
			Console.WriteLine(navn + " " + firma + " " + tlfNr);
			/*con = new OleDbConnection(connectionStr);
			cmd = new OleDbCommand();
			cmd.Connection = con;*/
			con.Open();             // ben forbindelsen til databasen
			OleDbTransaction trans = con.BeginTransaction();
			cmd.Transaction = trans;

			// Opbyg SQL-kommando
			cmd.CommandText = "INSERT INTO Gaest(GaestId, Navn, Firma, TelefonNr, aktiv) VALUES (@gaestId, @navn, @Firma, @tlfNr, @aktiv)";
			
			cmd.Parameters.Clear();
			cmd.Parameters.Add("@gaestId", OleDbType.Integer).Value = this.NaesteGaesteId++;
			cmd.Parameters.Add("@navn", OleDbType.VarChar).Value = navn;
			cmd.Parameters.Add("@firma", OleDbType.VarChar).Value = firma;
			cmd.Parameters.Add("@tlfNr", OleDbType.VarChar).Value = tlfNr;
			cmd.Parameters.Add("@aktiv", OleDbType.Boolean).Value = true;
			Console.WriteLine(cmd.CommandText);
			    
			try 
			{
				cmd.ExecuteNonQuery();   // Udfr kommando (SQL-streng)
				trans.Commit();
			}
			catch
			{
				Console.WriteLine("FEJL 40!!!!");
				trans.Rollback();
				con.Close();
				return false;
			}
			con.Close();             // Luk forbindelsen
			return true;
		}

		public bool SletGaest(string tlfNr)
		{
			/*con = new OleDbConnection(connectionStr);
			cmd = new OleDbCommand();
			cmd.Connection = con;*/
			con.Open();             // ben forbindelsen til databasen
			OleDbTransaction trans = con.BeginTransaction();
			cmd.Transaction = trans;

			// Opbyg SQL-kommando
			cmd.CommandText = "UPDATE Gaest SET aktiv = @aktiv WHERE TelefonNr = @tlfNr ";
			
			cmd.Parameters.Clear();
			cmd.Parameters.Add("@aktiv", OleDbType.Boolean).Value = false;
			cmd.Parameters.Add("@tlfNr", OleDbType.VarChar).Value = tlfNr;
			    
			try 
			{
				cmd.ExecuteNonQuery();   // Udfr kommando (SQL-streng)
				trans.Commit();
			}
			catch
			{
				trans.Rollback();
				con.Close();
				return false;
			}
			con.Close();             // Luk forbindelsen
			return true;
		}

		public bool OpdaterGaest(string navn, string firma, string tlfNr, string nytTlfNr)
		{
			/*con = new OleDbConnection(connectionStr);
			cmd = new OleDbCommand();
			cmd.Connection = con;*/
			con.Open();
			OleDbTransaction trans = con.BeginTransaction();
			cmd.Transaction = trans;

			// Opbyg SQL-kommandoer
			try
			{
				cmd.CommandText = "UPDATE Gaest SET Navn = @navn WHERE TelefonNr = @tlfNr ";
				cmd.Parameters.Clear();
				cmd.Parameters.Add("@navn", OleDbType.VarChar).Value = navn;
				cmd.Parameters.Add("@tlfNr", OleDbType.VarChar).Value = tlfNr;

				cmd.ExecuteNonQuery();
			
				cmd.CommandText = "UPDATE Gaest SET Firma = @firma WHERE TelefonNr = @tlfNr ";
				cmd.Parameters.Clear();
				cmd.Parameters.Add("@firma", OleDbType.VarChar).Value = firma;
				cmd.Parameters.Add("@tlfNr", OleDbType.VarChar).Value = tlfNr;
				cmd.ExecuteNonQuery();

				cmd.CommandText = "UPDATE Gaest SET TelefonNr = @nytTlfNr WHERE TelefonNr = @tlfNr ";
				cmd.Parameters.Clear();
				cmd.Parameters.Add("@navn", OleDbType.VarChar).Value = nytTlfNr;
				cmd.Parameters.Add("@tlfNr", OleDbType.VarChar).Value = tlfNr;
				cmd.ExecuteNonQuery(); 

				trans.Commit();
			}
			catch
			{
				trans.Rollback();
				con.Close();
				return false;
			}

			con.Close();             // Luk forbindelsen
			return true;
		}
		#endregion

		#region Metoder til Vrelse
		public bool GemVaerelse(int vNr, int type, bool ikkeRyger, bool handicap)
		{
			/*con = new OleDbConnection(connectionStr);
			cmd = new OleDbCommand();
			cmd.Connection = con;*/
			con.Open();            
			OleDbTransaction trans = con.BeginTransaction();
			cmd.Transaction = trans;

			// Opbyg SQL-kommando
			cmd.CommandText = "INSERT INTO Vaerelse(Vaerelsesnr, Type, Ikkeryger, Handicap, Aktiv) VALUES (@vNr, @type, @ikkeRyger, @handicap, @aktiv)";
			
			cmd.Parameters.Clear();
			cmd.Parameters.Add("@vNr", OleDbType.Integer).Value = vNr;
			cmd.Parameters.Add("@type", OleDbType.Integer).Value = type;
			cmd.Parameters.Add("@ikkeRyger", OleDbType.Boolean).Value = ikkeRyger;
			cmd.Parameters.Add("@handicap", OleDbType.Boolean).Value = handicap;
			cmd.Parameters.Add("@aktiv", OleDbType.Boolean).Value = true;
						    
			try 
			{
				cmd.ExecuteNonQuery();   // Udfr kommando (SQL-streng)
				trans.Commit();
			}
			catch //(System.Data.OleDb.OleDbException ex)
			{
				trans.Rollback();
				Console.ReadLine();
				return false;
			}
			con.Close();             // Luk forbindelsen
			return true;
		}

		
		
		public IVaerelse FindVaerelse(int nr)
		{
			/*con = new OleDbConnection(connectionStr);
			cmd = new OleDbCommand();
			cmd.Connection = con;*/
			con.Open();
			cmd.CommandText = "SELECT v.vaerelsesnr, v.type, v.ikkeryger, v.handicap, vt.pris FROM Vaerelse AS v, Vaerelsestype AS vt WHERE v.Vaerelsesnr = @nummer AND v.type = vt.vaerelsestype;";
			cmd.Parameters.Clear();
			cmd.Parameters.Add("@nummer", OleDbType.Integer).Value = nr;
			OleDbDataReader reader;
			reader = cmd.ExecuteReader();
			reader.Read();
			int nummer = 0;
			int type = -1;
			int pris = -1;
			bool ikkeryger = true;
			bool handicap = false;
			try 
			{
				nummer = reader.GetInt32(0);
				type = reader.GetInt32(1);
				ikkeryger = reader.GetBoolean(2);
				handicap = reader.GetBoolean(3);
				pris = reader.GetInt32(4);
			}
			catch (Exception)
			{
				//Console.WriteLine(ex.ToString());
				return null;
			}
			
			/*Console.WriteLine("Nste gsteId er: {0}",this.NaesteGaesteId);
			Console.WriteLine("Nste bookingId er: {0}",this.NaesteBookingId);
			Console.WriteLine("Nste bookingDagId er: {0}",this.NaesteBookingDagId);*/
			con.Close();
			return new Varelse(nummer,type,ikkeryger,handicap,pris);
		}

		public bool SletVaerelse(int vNr)
		{
			/*con = new OleDbConnection(connectionStr);
			cmd = new OleDbCommand();
			cmd.Connection = con;*/
			con.Open();            
			OleDbTransaction trans = con.BeginTransaction();
			cmd.Transaction = trans;

			// Opbyg SQL-kommando
			cmd.CommandText = "UPDATE Vaerelse SET aktiv = @aktiv WHERE VaerelsesNr = @vNr ";
			
			cmd.Parameters.Clear();
			cmd.Parameters.Add("@aktiv", OleDbType.Boolean).Value = false;
			cmd.Parameters.Add("@vNr", OleDbType.VarChar).Value = vNr;
			    
			try 
			{
				cmd.ExecuteNonQuery();   // Udfr kommando (SQL-streng)
				trans.Commit();
			}
			catch
			{
				trans.Rollback();
				return false;
			}
			con.Close();             // Luk forbindelsen
			return true;
		}
	
		public bool OpdaterVaerelse(int vNr, int type, bool ikkeRyger, bool handicap)
		{
			/*con = new OleDbConnection(connectionStr);
			cmd = new OleDbCommand();
			cmd.Connection = con;*/
			con.Open();
			OleDbTransaction trans = con.BeginTransaction();
			cmd.Transaction = trans;

			// Opbyg SQL-kommandoer
			try
			{
				cmd.CommandText = "UPDATE Vaerelse SET Type = @type WHERE VaerelsesNr = @vNr ";
				cmd.Parameters.Clear();
				cmd.Parameters.Add("@navn", OleDbType.Integer).Value = type;
				cmd.Parameters.Add("@vNr", OleDbType.Integer).Value = vNr;

				cmd.ExecuteNonQuery();
			
				cmd.CommandText = "UPDATE Vaerelse SET IkkeRyger = @ikkeRyger WHERE VaerelsesNr = @vNr ";
				cmd.Parameters.Clear();
				cmd.Parameters.Add("@ikkeRyger", OleDbType.Boolean).Value = ikkeRyger;
				cmd.Parameters.Add("@vNr", OleDbType.Integer).Value = vNr;
				cmd.ExecuteNonQuery();

				cmd.CommandText = "UPDATE Vaerelse SET Handicap = @handicap WHERE VaerelsesNr = @vNr ";
				cmd.Parameters.Clear();
				cmd.Parameters.Add("@handicap", OleDbType.Boolean).Value = handicap;
				cmd.Parameters.Add("@vNr", OleDbType.Integer).Value = vNr;
				cmd.ExecuteNonQuery();

				trans.Commit();
			}
			catch
			{
				trans.Rollback();
				return false;
			}

			con.Close();             // Luk forbindelsen
			return true;
		}
		#endregion

		#endregion
	}
}
